# Copyright 2022 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# type: ignore
from typing import Any

import pytest

from ckms import jose
from ckms.core.models import JSONWebSignature
from ckms.jose import PayloadCodec
from ckms.types import JSONWebToken
from ckms.types import MalformedPayload


@pytest.mark.asyncio
async def test_encrypt_fixed_algorithm(codec: jose.PayloadCodec):
    alg = codec.get_key_algorithm('RSA-OAEP')
    assert alg is not None
    payload = codec.encode({'foo': 'bar'}, encrypters=['RSA-OAEP'])
    token = await payload.serialize()
    header, claims = codec.introspect(token)
    assert header.alg == alg


@pytest.mark.asyncio
async def test_sign_fixed_algorithm(codec: jose.PayloadCodec):
    alg = codec.get_key_algorithm('RS256')
    payload = codec.encode({'foo': 'bar'}, signers=['RS256'])
    token = await payload.serialize()
    header, claims = codec.introspect(token)
    assert header.alg == alg


@pytest.mark.asyncio
async def test_introspect_binary(codec: jose.PayloadCodec):
    payload = codec.encode(b'Hello world!', signers=['HS256'])
    token = str.encode(await payload.serialize())
    _, claims = codec.introspect(token)
    assert claims is None


@pytest.mark.asyncio
async def test_introspect_jws_with_model(codec: jose.PayloadCodec):
    class CustomClaimSet(JSONWebToken):
        foo: str

    claims: CustomClaimSet
    payload = codec.encode(
        JSONWebToken.strict(iss='foo', aud='foo', foo='foo', ttl=30),
        signers=['HS256']
    )
    token = str.encode(await payload.serialize())
    _, claims = codec.introspect(token, model=CustomClaimSet)
    assert isinstance(claims, CustomClaimSet), type(claims)
    assert claims is not None
    assert claims.foo == 'foo'


@pytest.mark.asyncio
async def test_introspect_jws(codec: jose.PayloadCodec):
    payload = codec.encode(
        JSONWebToken.strict(iss='foo', aud='foo', foo='foo', ttl=30),
        signers=['HS256']
    )
    token = str.encode(await payload.serialize())
    _, claims = codec.introspect(token)
    assert claims is not None
    assert claims.iss == 'foo'


@pytest.mark.asyncio
async def test_codec_encode_claimset(codec: jose.PayloadCodec):
    payload = await codec.encode(JSONWebToken.strict(iss='foo', aud='foo', ttl=30), signers=['HS256'])
    jwt = await codec.decode(payload)
    assert jwt.aud == {'foo'}
    assert jwt.iss == 'foo'


@pytest.mark.asyncio
async def test_codec_encode_dict(codec: jose.PayloadCodec):
    payload = await codec.encode({'foo': 1}, signers=['HS256'])
    jwt = await codec.decode(payload)
    assert jwt.extra['foo'] == 1


@pytest.mark.asyncio
async def test_codec_encode_jws_with_overriden_typ(codec: jose.PayloadCodec):
    signed = await codec.encode(JSONWebToken(), signers=['HS256'], content_type="at+jwt")
    jwt = await codec.decode(signed, accept={"at+jwt"})
    assert isinstance(jwt, JSONWebToken)


@pytest.mark.asyncio
async def test_codec_encode_jws_with_overriden_can_deserialize_jwt(codec: jose.PayloadCodec):
    signed = await codec.encode(JSONWebToken(), signers=['HS256'], content_type="at+jwt")
    jwt = await codec.decode(signed, accept={"at+jwt"})
    assert isinstance(jwt, JSONWebToken), repr(jwt)


@pytest.mark.asyncio
async def test_codec_encode_and_sign_jwt(codec):
    jwt = JSONWebToken(foo=1)
    payload = await codec.encode(jwt, signers=['HS256'])
    decoded = await codec.decode(payload)
    assert isinstance(decoded, JSONWebToken), repr(decoded)


@pytest.mark.asyncio
async def test_codec_decode_jws(codec, encoded_jwt, jwt):
    payload = await codec.decode(encoded_jwt)
    assert isinstance(payload, JSONWebToken)


@pytest.mark.asyncio
async def test_codec_decode_jws_jwt(codec, encoded_jwt, jwt):
    payload = await codec.decode(encoded_jwt)
    assert jwt.extra.get('foo') == payload.extra.get('foo')


@pytest.mark.asyncio
async def test_codec_decode_jws_payload(codec, encoded_payload, payload):
    decoded = await codec.decode(encoded_payload)
    assert decoded == payload


@pytest.mark.asyncio
async def test_codec_decode_jwe_jwt(codec, encrypted_jwt, jwt):
    decrypted = await codec.decode(encrypted_jwt)
    assert isinstance(jwt, JSONWebToken)
    assert jwt.extra.get('foo') == decrypted.extra.get("foo")


@pytest.mark.asyncio
async def test_codec_decode_jwe_payload(codec, encrypted_payload, payload):
    decrypted = await codec.decode(encrypted_payload)
    assert decrypted == payload


@pytest.mark.asyncio
async def test_deserialize_nested_jws(codec: PayloadCodec):
    jwe = await codec.encode(
        JSONWebToken(foo=1),
        signers=['HS256'],
        encrypters=['RSA-OAEP']
    )
    jwt = await codec.decode(jwe)
    assert isinstance(jwt, JSONWebToken)


@pytest.mark.asyncio
async def test_deserialize_jws_payload(keychain, codec):
    jws = await codec.encode(b"Hello world!", signers=['HS256'])
    assert await codec.decode(jws) == b"Hello world!"


@pytest.mark.asyncio
async def test_deserialize_jwe_payload(keychain, codec):
    jwe = await codec.encode(b"Hello world!", encrypters=['RSA-OAEP'])
    assert await codec.decode(jwe) == b"Hello world!"



@pytest.mark.parametrize("payload", [
    b'Hello world!',
    {'foo': 'bar'}
])
@pytest.mark.parametrize("encrypters", [
    None,
    ['RSA-OAEP'],
    ['RSA-OAEP', 'RSA-OAEP-256'],
])
@pytest.mark.parametrize("signers", [
    ['HS256'],
    ['HS256', 'HS384'],
])
@pytest.mark.asyncio
async def test_deserialize_to_jws(
    payload: bytes | dict[str, Any],
    codec: PayloadCodec,
    signers: list[str],
    encrypters: list[str] | None
):
    token = await codec.encode(
        payload=payload,
        signers=signers,
        encrypters=encrypters
    )
    jws = await codec.jws(token)
    assert isinstance(jws, JSONWebSignature)


@pytest.mark.parametrize("payload", [
    b'Hello world!',
    {'foo': 'bar'}
])
@pytest.mark.parametrize("encrypters", [
    None,
    ['RSA-OAEP'],
    ['RSA-OAEP', 'RSA-OAEP-256'],
])
@pytest.mark.parametrize("signers", [
    ['HS256'],
    ['HS256', 'HS384'],
])
@pytest.mark.asyncio
async def test_deserialize_to_jwt(
    payload: bytes | dict[str, Any],
    codec: PayloadCodec,
    signers: list[str],
    encrypters: list[str] | None
):
    token = await codec.encode(
        payload=payload,
        signers=signers,
        encrypters=encrypters
    )
    if isinstance(payload, bytes):
        with pytest.raises(MalformedPayload):
            await codec.jwt(token)
    else:
        jws, claims = await codec.jwt(token)
        assert isinstance(jws, JSONWebSignature)
        assert isinstance(claims, JSONWebToken)


# Copyright 2022 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import ipaddress
from typing import Any

import pydantic
from cryptography import x509
from cryptography.hazmat.primitives.serialization import Encoding
from cryptography.hazmat.primitives.asymmetric.types import PUBLIC_KEY_TYPES
from pyasn1.codec.der import encoder
from pyasn1_modules import rfc2986

from ckms.ext import asn1
from ckms.ext.rfc5280.types import SignatureDigestAlgorithmIdentifier
from ckms.types import JSONWebKey
from ckms.types import Signer
from .certificationrequestinfo import CertificationRequestInfo
from .dnsname import DNSName
from .ipaddressname import IPAddressName
from .uriname import URIName


class CertificationRequest(asn1.ASN1Model):
    __asn1spec__: type[rfc2986.CertificationRequest] = rfc2986.CertificationRequest

    signed_data: bytes = b''

    info: CertificationRequestInfo = pydantic.Field(
        default_factory=CertificationRequestInfo,
        alias='certificationRequestInfo'
    )

    digest_algorithm: SignatureDigestAlgorithmIdentifier | None = pydantic.Field(
        default=None,
        alias='signatureAlgorithm'
    )

    signature: asn1.fields.BitString = asn1.fields.BitString(None)

    @property
    def public_key(self) -> JSONWebKey:
        assert self.digest_algorithm is not None
        return JSONWebKey.frompublic(
            key=self.get_public_key(),
            alg=self.digest_algorithm.get_qualname(
                key=self.info.subject_public_key
            )
        )

    @classmethod
    def new(
        cls,
        public_key: PUBLIC_KEY_TYPES,
    ) -> 'CertificationRequest':
        """Instantiate a new :class:`CertificationRequest` instance. Set the
        `SubjectPublicKeyInfo` attribute and return the new instance.

        The `public_key` parameter is the public key of the signer. The caller
        must ensure that the actual signature is created using the corresponding
        private key.
        """
        return cls(
            certificationRequestInfo=CertificationRequestInfo.new(key=public_key)
        )

    @classmethod
    def parse_der(cls, value: bytes) -> 'CertificationRequest':
        return super().parse_der(value) # type: ignore

    @pydantic.root_validator(pre=True)
    def preprocess(cls, values: dict[str, Any]) -> dict[str, Any]:
        info = values.get('certificationRequestInfo')
        sig = values.get('signature')
        if info and sig:
            values['signed_data'] = encoder.encode(info)
        return values

    def asn1(self) -> rfc2986.CertificationRequest:
        obj = super().asn1()

        # If there was previously signed data (for example, when this
        # instance was parsed from an existing CSR), use that instead
        # of encoding from this child models.
        if self.signed_data:
            obj.setComponentByName( # type: ignore
                name='certificationRequestInfo',
                value=asn1.decode_der(
                    value=self.signed_data,
                    spec=rfc2986.CertificationRequestInfo()
                )
            )
        return obj # type: ignore

    def add_domain(self, value: str) -> None:
        """Add a domain name to the Subject Alternative Name (SAN)
        certificate extension.
        """
        self.info.extensions.add_san(DNSName(value=value))

    def add_ip(self, value: ipaddress.IPv4Address | str) -> None:
        """Add an IPv4 address to the Subject Alternative Name (SAN)
        certificate extension.
        """
        if isinstance(value, str):
            value = ipaddress.IPv4Address(value)
        self.info.extensions.add_san(IPAddressName(value=value))

    def add_uri(self, value: str) -> None:
        """Add an Uniform Resource Identifier (URI) to the Subject Alternative
        Name (SAN) certificate extension.
        """
        self.info.extensions.add_san(URIName(value=value))

    def can_sign(self, value: bool | None = None) -> bool:
        """Return a boolean indicating if the extension allows the
        verification of digital signatures.
        """
        return self.info.extensions.can_sign(value)

    def get_public_key(self) -> PUBLIC_KEY_TYPES:
        """Return the public key of the signer."""
        return self.info.subject_public_key.get_public_key()

    def nonrepudiable(self, value: bool | None = None) -> bool:
        """Return a boolean indicating if signatures made by the private
        key associated to the certificate public key are nonrepudiable.
        """
        return self.info.extensions.nonrepudiable(value)

    def pem(self) -> bytes:
        """Return a byte-sequence holding the PEM-encoded CSR."""
        csr = x509.load_der_x509_csr(self.der())
        return csr.public_bytes(Encoding.PEM)

    def verify(self, key: JSONWebKey | None = None) -> bool:
        """Verify that the signature was made using the included public
        key.
        """
        key = key or self.public_key
        if self.digest_algorithm is None:
            return False
        return key.verify(
            signature=self.info.subject_public_key.algorithm.normalize_signature(self.signature),
            message=self.signed_data
        )

    async def sign(self, signer: Signer) -> None:
        """Sign the :class:`CertificationRequest` with the provided `signer`."""
        if self.signed_data:
            raise ValueError("The CertificationRequest already has a signature")
        self.signed_data = self.info.der()
        self.digest_algorithm = SignatureDigestAlgorithmIdentifier.parse_obj({
            'algorithm': signer.get_digest_oid()
        })
        self.signature = asn1.fields.BitString(
            self.info.subject_public_key.algorithm.encode_signature(await signer.sign(self.signed_data))
        )


# Copyright 2022 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import inspect
import pathlib

from ckms.core.keychain import Keychain

import pytest

from ckms.pkix.types import CertificationRequest
from ckms.ext.rfc5280.types import Attribute
from ckms.ext.rfc5280.types import RelativeDistinguishedName


CURDIR: pathlib.Path = pathlib.Path(__file__).parent


@pytest.mark.parametrize("name", [
    "RS256",
    "RS384",
    "RS512",
    'ES256',
    #'ES256-384',
    #'ES256-512',
    'ES384',
    #'ES384-256',
    #'ES384-512',
    'ES512',
    #'ES512-256',
    #'ES512-512',
    'ES256K',
    #'ES256K-384',
    #'ES256K-512',
])
@pytest.mark.asyncio
async def test_sign_csr_simple(
    keychain: Keychain,
    name: str
):
    spec = keychain.get(name)
    csr = CertificationRequest.new(
        public_key=spec.get_public_key()
    )

    # Add some attributes.
    csr.add_uri('https://unimatrixone.io')
    csr.add_domain('unimatrixone.io')
    csr.add_ip('1.2.3.4')
    csr.nonrepudiable(True)
    csr.can_sign(True)
    csr.info.subject.extend([
        RelativeDistinguishedName([
            Attribute(
                type='2.5.4.6',
                value='NL'
            )
        ]),
        RelativeDistinguishedName([
            Attribute(
                type='2.5.4.7',
                value='Wezep'
            )
        ]),
        RelativeDistinguishedName([
            Attribute(
                type='2.5.4.8',
                value='Gelderland'
            )
        ]),
        RelativeDistinguishedName([
            Attribute(
                type='2.5.4.3',
                value='Cochise Ruhulessin'
            ),
            Attribute(
                type='2.5.4.42',
                value='Cochise'
            ),
            Attribute(
                type='2.5.4.43',
                value='C.Y.'
            ),
        ]),
    ])

    await csr.sign(spec)
    assert csr.signed_data
    assert csr.digest_algorithm
    is_valid = csr.verify()
    if inspect.isawaitable(is_valid):
        is_valid = await is_valid

    # Serialize and parse again to see if the signature is still
    # valid.
    der = csr.der()
    csr2 = CertificationRequest.parse_der(der)
    assert csr.info.der() == csr2.info.der()
    assert csr.signed_data == csr2.signed_data, csr2.signed_data
    is_valid = csr2.verify()
    if inspect.isawaitable(is_valid):
        is_valid = await is_valid
    assert is_valid


@pytest.mark.parametrize("fn", [
    'RS256.csr',
    'RS384.csr',
    'RS512.csr',
    'ES256.csr',
    'ES256-384.csr',
    'ES256-512.csr',
    'ES256K.csr',
    'ES256K-384.csr',
    'ES256K-512.csr',
    'ES384.csr',
    'ES384-256.csr',
    'ES384-512.csr',
    'ES512-256.csr',
    'ES512-384.csr',
    'ES512.csr',
])
@pytest.mark.asyncio
async def test_verify_from_subjectpublickeyinfo(fn: str):
    with open(CURDIR.joinpath(fn), 'rb') as f:
        csr = CertificationRequest.parse_der(f.read())
    is_valid = csr.verify()
    if inspect.isawaitable(is_valid):
        is_valid = await is_valid
    assert is_valid

    # Serialize and parse again to see if the signature is still
    # valid.
    der = csr.der()
    csr = CertificationRequest.parse_der(der)
    is_valid = csr.verify()
    if inspect.isawaitable(is_valid):
        is_valid = await is_valid
    assert is_valid



# Copyright 2022 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

DEFAULT_CONTENT_ENCRYPTION_ALGORITHM: str = 'A256GCM'

DEFAULT_EC_ENCRYPTION_ALGORITHM: str = 'ECDH-ES+A256KW'

DEFAULT_EC_ENCRYPTION_CURVE: str = 'P-256K'

DEFAULT_EC_SIGNING_ALGORITHM: str = 'ES384'

DEFAULT_EC_SIGNING_CURVE: str = 'P-384'

DEFAULT_ED_ENCRYPTION_ALGORITHM: str = 'ECDH-ES+A256KW'

DEFAULT_ED_ENCRYPTION_CURVE: str = 'X448'

DEFAULT_ED_SIGNING_CURVE: str = 'Ed448'

DEFAULT_RSA_ENCRYPTION_ALG: str = 'RSA-OAEP-256'

DEFAULT_RSA_SIGNING_ALG: str = 'PS256'

DEFAULT_SYMMETRIC_ENCRYPTION_ALGORITHM: str = 'A256GCMKW'

DEFAULT_SYMMETRIC_SIGNING_ALGORITHM: str = 'HS256'

DH_ALGORITHMS: set[str] = {
    'ECDH-ES',
    'ECDH-ES+A128KW',
    'ECDH-ES+A192KW',
    'ECDH-ES+A256KW',
}

ENCRYPTION_ALGORITHMS: set[str] = {
    #'A128CBC',
    #'A192CBC',
    #'A256CBC',
    'A128GCM',
    'A192GCM',
    'A256GCM',
}

KEYWRAP_ALGORITHMS: set[str] = {
    'A128KW',
    'A192KW',
    'A256KW',
    'A128GCMKW',
    'A192GCMKW',
    'A256GCMKW',
    'RSA1_5',
    'RSA-OAEP',
    'RSA-OAEP-256',
    'RSA-OAEP-384',
    'RSA-OAEP-512',
}

SIGNING_ALGORITHMS: set[str] = {
    'EdDSA',
    'ES256',
    'ES384',
    'ES512',
    'ES256K',
    'HS256',
    'HS384',
    'HS512',
    'RS256',
    'RS384',
    'RS512',
    'PS256',
    'PS384',
    'PS512',
}

ALGORITHMS: set[str] = DH_ALGORITHMS|ENCRYPTION_ALGORITHMS|KEYWRAP_ALGORITHMS|SIGNING_ALGORITHMS

ASYMMETRIC_SIGNING_ALGORITHMS: set[str] = {
    'EdDSA',
    'ES256',
    'ES384',
    'ES512',
    'ES256K',
    'RS256',
    'RS384',
    'RS512',
    'PS256',
    'PS384',
    'PS512',
}

ASYMMETRIC_ALGORITHMS: set[str] = {
    *ASYMMETRIC_SIGNING_ALGORITHMS,
    'ECDH-ES',
    'ECDH-ES+A128KW',
    'ECDH-ES+A192KW',
    'ECDH-ES+A256KW',
    'RSA1_5',
    'RSA-OAEP',
    'RSA-OAEP-256',
    'RSA-OAEP-384',
    'RSA-OAEP-512',
}

SYMMETRIC_ALGORITHMS: set[str] = ALGORITHMS - ASYMMETRIC_ALGORITHMS


# Copyright 2018 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# type: ignore
from typing import Any

import pydantic
from pyasn1_modules import rfc3161

from .pkifailureinfo import PKIFailureInfo
from .pkistatus import PKIStatus


class PKIStatusInfo(pydantic.BaseModel):
    status_code: PKIStatus = pydantic.Field(..., alias='status')
    status_message: str | None = pydantic.Field(None, alias='statusString')
    failure_reason: PKIFailureInfo | None = pydantic.Field(None, alias='failInfo')

    @pydantic.validator('status_code', pre=True)
    def validate_status_code(cls, value: int | rfc3161.PKIStatus | PKIStatus) -> int:
        if isinstance(value, rfc3161.PKIStatus):
            value = int(value)
        return value

    @pydantic.validator('status_message', pre=True)
    def validate_status_message(cls, value: str | rfc3161.PKIFreeText) -> str:
        if isinstance(value, rfc3161.PKIFreeText):
            value = str(value[0]) if value.isValue else None
        return value

    @pydantic.validator('failure_reason', pre=True)
    def validate_failure_reason(cls, value: int | rfc3161.PKIFailureInfo | PKIFailureInfo) -> str:
        if isinstance(value, rfc3161.PKIFailureInfo):
            value = int(value) if value.isValue else None
        return value

    def is_success(self) -> bool:
        """Return a boolean indicating if the status is a success."""
        return self.status_code in {PKIStatus.granted, PKIStatus.grantedWithMods}


import asyncio

import ckms
from ckms import jose


codec = jose.PayloadCodec()


async def main():
    # For this example, a key is generated. In a real-world use-case,
    # the key would be loaded from disk or reference an external key
    # management service (such as Google Cloud KMS or an HSM).
    # Be aware to NEVER use in production RSA keys with a bit length
    # lower than 2048 bits.
    await ckms.configure({
        'sig': {
            'provider': "local",
            'kty': "RSA",
            'bit_length': 2048
        },
        'enc': {
            'provider': "local",
            'kty': "RSA",
            'bit_length': 2048
        },
    })

    data = b'Hello world!'
    payload: jose.Payload = codec.encode(data)

    # Create a JWS using the `sig` key and `RS256`
    # algorithm.
    jws: jose.Payload = await payload.sign('RS256', 'sig')

    # Encrypt the JWS using the `enc` key and the default content
    # encryption algorithm (A256GCM).
    jwe: jose.Payload = await jws.encrypt('RSA-OAEP-256', 'enc')

    # Decoded the serialized JWE.
    decoded = await codec.decode(bytes(jwe))
    assert decoded == data


if __name__ == '__main__':
    asyncio.run(main())


# Copyright 2022 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import copy
from typing import Any

import pytest
import pytest_asyncio

import ckms.core
from ckms.core import Keychain
from ckms.jose import PayloadCodec
from ckms.jose.decoder import Decoder
from ckms.jose.encoder import Encoder
from ckms.types import JSONWebToken
from ckms.types import IKeychain


@pytest.fixture
def decoder(keychain: Keychain) -> Decoder:
    return Decoder(decrypter=keychain, verifier=keychain)


@pytest.fixture
def encoder(keychain: Keychain) -> Encoder:
    return Encoder(encrypter=keychain, signer=keychain)


@pytest.fixture
def codec(keychain: IKeychain):
    assert keychain is not None
    return PayloadCodec(
        signer=keychain,
        verifier=keychain,
        encrypter=keychain,
        decrypter=keychain
    )


@pytest.fixture
def payload() -> bytes:
    return b'Hello world!'


@pytest.fixture
def jwt() -> JSONWebToken:
    return JSONWebToken.parse_obj({'foo': 1})


@pytest_asyncio.fixture # type: ignore
async def encoded_jwt(codec: PayloadCodec, jwt: JSONWebToken) -> bytes:
    return str.encode(await codec.encode(jwt, signers=['HS256']))


@pytest_asyncio.fixture # type: ignore
async def encoded_payload(codec: PayloadCodec, payload: bytes) -> bytes:
    return str.encode(await codec.encode(payload, signers=['HS256']))


@pytest_asyncio.fixture # type: ignore
async def encrypted_jwt(codec: PayloadCodec, jwt: JSONWebToken) -> bytes:
    return str.encode(await codec.encode(jwt, encrypters=['RSA-OAEP']))


@pytest_asyncio.fixture # type: ignore
async def encrypted_payload(codec: PayloadCodec, payload: bytes) -> bytes:
    return str.encode(await codec.encode(payload, encrypters=['RSA-OAEP']))


import asyncio

import ckms


async def main():
    await ckms.configure({
        'example': {
            'provider': "local",
            'path': "pki/rsa.key"
        }
    })

    msg = b"Hello world!"
    sig = await ckms.sign(msg, using='example', algorithm='RS256')
    assert await ckms.verify(
        signature=sig,
        data=msg,
        using='example',
        algorithm='RS256'
    )

    assert not await ckms.verify(
        signature=sig,
        data=b'foo',
        using='example',
        algorithm='RS256'
    )


if __name__ == '__main__':
    asyncio.run(main())



# Copyright 2018 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Declares :class:`IHTTPResponse`"""
from typing import Any
from typing import Awaitable


class IHTTPResponse:
    content: bytes
    status_code: int

    def json(
        self,
        *args: Any,
        **kwargs: Any
    ) -> dict[str, Any] | list[Any] | Awaitable[dict[str, Any] | list[Any]]:
        raise NotImplementedError